@@ -1,7 +1,7 @@
 /* mpz_fdiv_q_ui -- Division rounding the quotient towards -infinity.
    The remainder gets the same sign as the denominator.
 
-Copyright (C) 1994, 1995, 1996, 1999 Free Software Foundation, Inc.
+Copyright 1994, 1995, 1996, 1999, 2001, 2002 Free Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
@@ -24,42 +24,69 @@ MA 02111-1307, USA. */
 #include "gmp-impl.h"
 
 unsigned long int
-#if __STDC__
 mpz_fdiv_q_ui (mpz_ptr quot, mpz_srcptr dividend, unsigned long int divisor)
-#else
-mpz_fdiv_q_ui (quot, dividend, divisor)
-     mpz_ptr quot;
-     mpz_srcptr dividend;
-     unsigned long int divisor;
-#endif
 {
-  mp_size_t dividend_size;
-  mp_size_t size;
-  mp_ptr quot_ptr;
-  mp_limb_t remainder_limb;
+  mp_size_t ns, nn, qn;
+  mp_ptr np, qp;
+  mp_limb_t rl;
 
   if (divisor == 0)
     DIVIDE_BY_ZERO;
 
-  dividend_size = dividend->_mp_size;
-  size = ABS (dividend_size);
-
-  if (quot->_mp_alloc < size)
-    _mpz_realloc (quot, size);
-
-  quot_ptr = quot->_mp_d;
-
-  remainder_limb = mpn_divmod_1 (quot_ptr, dividend->_mp_d, size,
-				   (mp_limb_t) divisor);
-
-  if (remainder_limb != 0 && dividend_size < 0)
+  ns = SIZ(dividend);
+  if (ns == 0)
     {
-      mpn_incr_u (quot_ptr, (mp_limb_t) 1);
-      remainder_limb = divisor - remainder_limb;
+      SIZ(quot) = 0;
+      return 0;
     }
 
-  size -= size != 0 && quot_ptr[size - 1] == 0;
-  quot->_mp_size = dividend_size >= 0 ? size : -size;
+  nn = ABS(ns);
+  MPZ_REALLOC (quot, nn);
+  qp = PTR(quot);
+  np = PTR(dividend);
 
-  return remainder_limb;
+#if GMP_NAIL_BITS != 0
+  if (divisor > GMP_NUMB_MAX)
+    {
+      mp_limb_t dp[2], rp[2];
+
+      if (nn == 1)		/* tdiv_qr requirements; tested above for 0 */
+	{
+	  qp[0] = 0;
+	  rl = np[0];
+	  qn = 1;		/* a white lie, fixed below */
+	}
+      else
+	{
+	  dp[0] = divisor & GMP_NUMB_MASK;
+	  dp[1] = divisor >> GMP_NUMB_BITS;
+	  mpn_tdiv_qr (qp, rp, (mp_size_t) 0, np, nn, dp, (mp_size_t) 2);
+	  rl = rp[0] + (rp[1] << GMP_NUMB_BITS);
+	  qn = nn - 2 + 1; 
+	}
+
+      if (rl != 0 && ns < 0)
+	{
+	  mpn_incr_u (qp, (mp_limb_t) 1);
+	  rl = divisor - rl;
+	}
+
+      qn -= qp[qn - 1] == 0; qn -= qn != 0 && qp[qn - 1] == 0;
+    }
+  else
+#endif
+    {
+      rl = mpn_divrem_1 (qp, (mp_size_t) 0, np, nn, (mp_limb_t) divisor);
+
+      if (rl != 0 && ns < 0)
+	{
+	  mpn_incr_u (qp, (mp_limb_t) 1);
+	  rl = divisor - rl;
+	}
+
+      qn = nn - (qp[nn - 1] == 0);
+    }
+
+  SIZ(quot) = ns >= 0 ? qn : -qn;
+  return rl;
 }
